unit e_logo;

interface

uses OpenGL, e_models, e_textures, e_types, e_utils, e_sound, e_global;

const
  logo_time = 000;

type
  TLogoLetter = record
    pos   : single;
    speed : single;
  end;

var
  logo_count   : single = 0;
  logo_letters : array[1..4] of TLogoLetter;

procedure LogoInitialize;
procedure LogoRender;
procedure LogoUpdate(count : single);

implementation

procedure LogoInitialize;
var
i : integer;
begin
if not render_logo then logo_count := logo_time;

for i := 1 to 4 do
  with logo_letters[i] do
    begin
      pos   := 25 + i * 10;
      speed := -0.02;
    end;
end;

procedure LogoRender;
var
i : integer;
s : single;
begin
glDisable(GL_LIGHTING);
glDisable(GL_DEPTH_TEST);
glEnable(GL_BLEND);
glEnable(GL_TEXTURE_2D);

glBlendFunc(GL_SRC_ALPHA, GL_ONE);

s := 6 - logo_count / 1000;

for i := 1 to 4 do
  with logo_letters[i] do
    begin
      SetTexture(texture_map);
      glColor4f(0.5, 0, 0, s);
      glPushMatrix;
      glTranslatef(0.2*pos, 0, 0);
      glScalef(0.2, 0.2, 0.2);
      case i of
        1 : glCallList(model_quad);
        2 : glCallList(model_quad);
        3 : glCallList(model_quad);
        4 : glCallList(model_quad);
      end;

      SetTexture(texture_map);
      glColor4f(0.1, 0.1, 0.1, s);
      glScalef(2, 2, 1);
      glRotatef(i * 5 + logo_count * 0.1, 0, 0, 1);
      glCallList(model_quad);

      glPopMatrix;
    end;

glPushMatrix;
glTranslatef(3, -2, 0);
glScalef(194 * 0.02, 59 * 0.02, 0);
glColor4f(1, 1, 1, s * (logo_count - 3500) * 0.0005);

glCallList(model_quad);
glPopMatrix;

glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
end;

procedure LogoUpdate(count : single);
var
i, j : integer;
begin
if logo_count < logo_time then
  logo_count := logo_count + count
else exit;

for i := 1 to 4 do
  with logo_letters[i] do
    begin
      pos := pos + speed * count;
      if pos < 13.5 then
        Aspire(speed, 0, count * 0.00002);

      for j := i + 1 to 4 do
        if abs(pos - logo_letters[j].pos) < 4.3 then
          begin
            speed := logo_letters[j].speed;
          end;
    end;
end;

end.
