unit e_main;

interface

uses Windows, OpenGL, e_render, e_global, e_sound, e_types, e_models, e_particles, e_music,
     e_utils, e_screen, e_game, e_textures, e_font, e_logo,
     e_console;

procedure GameFinish;     
function  GameStart(radius : single) : boolean;
procedure Initialize;
procedure Deinitialize;
procedure Update(count : single);
procedure Render;

var
  menu_active : boolean;

implementation  

procedure ResetTime;
begin
count_fps := 0;
count_last := GetTickCount;
count_sec := count_last;
time := 0;
time_scale := 1;
end;

procedure GameFinish;
begin
game.Save;
game_started := false;
game.Free;
particles.Free;
GameStart(35);
end;

function GameStart(radius : single) : boolean;
begin
result := false;

if game_started then GameFinish;

particles    := TParticlesEngine.Create;
game         := TGame.Create(radius);

ResetTime;
game_started := true;

result := true;
end;

procedure Initialize;
begin
menu_active := true;
ModelsInitialize;
Console.Write('Models');
TexturesInitialize;
Console.Write('Textures');
SoundInitialize;
Console.Write('Sound');
LogoInitialize;
Console.Write('Intro');
font := TFont.Create;
Console.Write('Font');
clear_color := GetColor(0.3, 0.4, 0.5, 1);

GameStart(35);

Console.Write('Game');

menu_active := false;
end;

procedure Deinitialize;
begin
GameFinish;
SoundDeinitialize;
end;

procedure Update(count : single);
begin
console.Update(count);
//count := count * time_scale;
time := time + count;

texture_time := time * 0.0005;

MusicUpdate(count);
SoundUpdate(count);

LogoUpdate(count);
if logo_count < logo_time then exit;

if (not menu_active) or (game.game_over) then
  if (not game_pause) then
    game.Update(count);
end;

procedure CountFPS;
begin
count_new := GetTickCount;

inc(count_fps);
if (count_new - count_sec) >= 1000 then
  begin
    count_sec := count_sec + 1000;
    fps := count_fps;
    count_fps := 0;
  end;
end;

procedure RenderStart;
begin
CountFPS;

glMatrixMode(GL_PROJECTION);
glLoadIdentity;

glViewPort(0, 0, screen.width, screen.height);

with clear_color do glClearColor(r, g, b, a);
glClear(GL_COLOR_BUFFER_BIT or GL_DEPTH_BUFFER_BIT);
end;

procedure RenderFinish;
begin
glMatrixMode(GL_MODELVIEW);
glLoadIdentity;
SwapBuffers(DC);
count_last := count_new;
end;

procedure RenderMenu;
begin
glMatrixMode(GL_PROJECTION);
glLoadIdentity;
glOrtho(0, 1, screen.aspect, 0, -1, 1);
glMatrixMode(GL_MODELVIEW);
glLoadIdentity;
end;

procedure Render;
begin
RenderStart;

if logo_count < logo_time then
  LogoRender
else  
  begin
    e_render.Render;
    if menu_active then
      RenderMenu;
  end;

Console.Render;

RenderFinish;
end;

end.
