unit e_models;

interface

uses OpenGL, e_global, e_types, e_utils;

const
  max_data          = 2000;
  max_frames        = 4;

var
  model_quad, model_sphere : gluint;
  model_ground : gluint;
  quadric : GLUquadricObj;
  models_quality : integer = 24;

procedure ModelsInitialize;

type
  DataS = array[0..max_data, 0..2] of single;
  PDataS = ^DataS;

  DataI = array[0..max_data, 0..5] of integer;
  PDataI = ^DataI;

  RModel = record
    faces : DataI;
    texture_vertexes, vertexes_normals, faces_normals : DataS;
    vertexes : array[0..max_data, 0..max_frames, 0..2] of single;
  end;

var
  load_model : RModel;
  total_frames,
  total_vertexes,
  total_faces,
  total_texture_vertexes : integer;

implementation

//  .    =)
function LoadModel(file_name : string) : RModel;
var
f : file of byte;
i1, i2, i3, i, a : integer;
s : array[0..2] of char;
begin
assignfile(f, file_name);
reset(f);

BlockRead(f, s, sizeof(char) * 3);

// ,        
// -      
// ... ,  =)
if s[0] + s[1] + s[2] = 'VFM' then
  with result do
  begin
    BlockRead(f, total_frames, 2);
    BlockRead(f, total_vertexes, 2);
    BlockRead(f, total_texture_vertexes, 2);
    BlockRead(f, total_faces, 2);

    for i1 := 0 to total_vertexes-1 do
      for i2 := 0 to total_frames do
        for i3 := 0 to 2 do
          BlockRead(f, vertexes[i1, i2, i3], 4);

    for i1 := 0 to total_vertexes-1 do
      for i2 := 0 to 2 do
        BlockRead(f, vertexes_normals[i1, i2], 4);

    for i1 := 0 to total_texture_vertexes-1 do
      for i2 := 0 to 1 do
        BlockRead(f, texture_vertexes[i1, i2], 4);

    for i1 := 0 to total_faces-1 do
      for i2 := 0 to 5 do
        BlockRead(f, faces[i1, i2], 2);
  end;
closefile(f);
end;

//   
function PrepareModel(file_name : string) : gluint;
var
i1, i2, a : integer;
begin
file_name := file_name;

if not FileExists(file_name) then exit;
load_model := LoadModel(file_name);

result := glGenLists(1);
glNewList(result, GL_COMPILE);
glBegin(GL_TRIANGLES);
                     
with load_model do
for i1 := 0 to total_faces-1 do
  for i2 := 2 downto 0 do
    begin
      a := faces[i1, i2 + 3];
      glTexCoord2d(texture_vertexes[a, 0],
                   texture_vertexes[a, 1]);
      a := faces[i1, i2];
      glNormal3f(vertexes_normals[a, 0],
                 vertexes_normals[a, 1],
                 vertexes_normals[a, 2]);
      glVertex3f(vertexes[a, 0, 0],
                 vertexes[a, 0, 1],
                 vertexes[a, 0, 2]);
    end;      

glEnd;
glEndList;
end;

function PrepareQuad : gluint;
begin
result := glGenLists(1);
glNewList(result, GL_COMPILE);

glBegin(GL_QUADS);
glNormal3f(0.0, 0.0, 1.0);
glTexCoord2f(0.0, 0.0); glVertex2f(-0.5, -0.5);
glTexCoord2f(1.0, 0.0); glVertex2f( 0.5, -0.5);
glTexCoord2f(1.0, 1.0); glVertex2f( 0.5,  0.5);
glTexCoord2f(0.0, 1.0); glVertex2f(-0.5,  0.5);
glEnd;

glEndList;
end;

function PrepareSphere : gluint;
begin
result := glGenLists(1);
glNewList(result, GL_COMPILE);

gluQuadricNormals(quadric, GL_FLAT);
gluQuadricTexture(quadric, true);

gluQuadricOrientation(quadric, GLU_OUTSIDE);
gluSphere(quadric, 1, models_quality, models_quality);

glEndList;
end;

procedure ModelsInitialize;
begin
quadric          := gluNewQuadric;

model_quad       := PrepareQuad;
model_sphere     := PrepareSphere;
model_ground     := PrepareModel('models/ground.vfm');

gluDeleteQuadric(quadric);
end;

end.
