unit e_system;

interface

uses Windows, Messages, OpenGL, e_global, e_screen, e_main, e_font, e_game, e_console,
     e_about, e_utils, e_types, e_particles, e_sound, e_logo, inifiles;

function  oglInitialization : boolean;
procedure oglDeinitialization;
function  WndProc(hwnd: DWORD; message: UINT; wParam: Longint; lParam: LongInt): LongInt; stdcall;
function  CreateWindow: HWND;
function  GetTimer: integer;
function  ProgramInitialization : boolean;
procedure ProgramDeInitialization;

implementation

procedure OptionsLoad;
var
ini : TIniFile;
n : integer;
begin
ini := TIniFile.Create(ExtractPath(ParamStr(0)) + 'options.ini');

screen.resolutions.number := ini.ReadInteger('Video', 'Resolution', 1);
screen.fullscreen         := ini.ReadBool('Video',    'Fullscreen', false);
screen_frequency          := ini.ReadInteger('Video', 'Frequency',  75);
screen.vsync              := ini.ReadBool('Video',    'VSync',      true);

volume_sound              := ini.ReadInteger('Audio', 'Sounds',     8);
volume_music              := ini.ReadInteger('Audio', 'Music',      8);

render_logo               := ini.ReadBool('Advanced', 'Intro',      false);

ini.Free;   
end;

procedure OptionsSave;
var
ini : TIniFile;
begin
ini := TIniFile.Create(ExtractPath(ParamStr(0)) + 'options.ini');

ini.WriteInteger('Video',    'Resolution', screen.resolutions.number);
ini.WriteBool('Video',       'Fullscreen', screen.fullscreen);
ini.WriteInteger('Video',    'Frequency',  screen_frequency);
ini.WriteBool('Video',       'VSync',      screen.vsync);

ini.WriteInteger('Audio',    'Sounds',     volume_sound);
ini.WriteInteger('Audio',    'Music',      volume_music);

ini.WriteBool('Advanced',    'Intro',      render_logo);

ini.Free;
end;

function oglInitialization : boolean;
var
 pfd     : PIXELFORMATDESCRIPTOR;
 iFormat : integer;
begin
Result := false;
DC := GetDC(handle);

if DC = 0 then Exit;
FillChar(pfd, SizeOf(pfd), 0);
with pfd do
  begin
   nSize      := SizeOf(TPIXELFORMATDESCRIPTOR);
   nVersion   := 1;
   dwFlags    := PFD_DRAW_TO_WINDOW or
                 PFD_SUPPORT_OPENGL or
                 PFD_DOUBLEBUFFER;
   iPixelType := PFD_TYPE_RGBA;
   cColorBits := 32;
   cDepthBits := 32;
   iLayerType := PFD_MAIN_PLANE;
  end;

iFormat := ChoosePixelFormat(DC, @pfd);
if iFormat = 0 then Exit;

if not SetPixelFormat(DC, iFormat, @pfd) then Exit;

RC := wglCreateContext(DC);
if RC = 0 then Exit;
if not wglMakeCurrent(DC, RC) then Exit;

Result := true;
end;

procedure oglDeinitialization;
begin
wglMakeCurrent(DC, 0);
wglDeleteContext(RC);
ReleaseDC(Handle, DC);
DestroyWindow(Handle);
end;

function WndProc(hwnd: DWORD; message: UINT; wParam: Longint; lParam: LongInt): LongInt; stdcall;
var
i, j : integer;
begin
case message of
  WM_QUERYENDSESSION,
  WM_CLOSE            : quit := true;
  WM_KEYDOWN          : begin
                          if Console.Char(chr(wParam)) then
                            exit;

                          if logo_count < logo_time then
                            begin
                              logo_count := logo_time;
                              exit;
                            end;

                          game.started := true;
                        end;
  WM_MOUSEMOVE        : begin
                          if logo_count < logo_time then
                            exit;

                          i := LOWORD(lParam);
                          j := HIWORD(lParam);

                          mouse_system.x := mouse_system.x + mouse_sens * (i - mouse_system_old.x) * screen.width / 640;
                          mouse_system.y := mouse_system.y + mouse_sens * (j - mouse_system_old.y) * screen.height / 480;
                          if mouse_system.x < 0 then mouse_system.x := 0;
                          if mouse_system.x > screen.width then mouse_system.x := screen.width;
                          if mouse_system.y < 0 then mouse_system.y := 0;
                          if mouse_system.y > screen.height then mouse_system.y := screen.height;
                          mouse_system_old := GetPoint(i, j);
                          i := round(mouse_system.x);
                          j := round(mouse_system.y);
                          
                          i := LOWORD(lParam);
                          j := HIWORD(lParam);

                          mouse_game.x :=  camera_pos.x + game_size * (i / screen.width * 2 - 1);
                          mouse_game.y := (camera_pos.y + game_size * (-j / screen.height * 2 + 1)) / screen.aspect;
                        end;
  WM_LBUTTONDOWN, WM_RBUTTONDOWN
                      : begin
                          if message = WM_RBUTTONDOWN then mouse_right_down := true;
                          
                          if logo_count < logo_time then
                            exit;

                          game.started := true;

                          mouse_down := true;
                        end;
  WM_LBUTTONUP, WM_RBUTTONUP
                        : begin
                          if logo_count < logo_time then
                            exit;

                          mouse_down := false;
                        end;
  WM_DESTROY          : quit := true;
end;

Result := DefWindowProc(hwnd, message, wParam, lParam);
end;

function CreateWindow: HWND;
var
 wndclass : WNDCLASSEX;
begin
ZeroMemory(@wndclass, sizeof(WNDCLASSEX));
with wndclass do
  begin
    cbSize        := sizeof(WNDCLASSEX);
    style         := CS_HREDRAW or CS_VREDRAW or CS_OWNDC;
    lpfnWndProc   := @WndProc;
    cbClsExtra    := 0;
    cbWndExtra    := 0;
    hInstance     := 0;
    hIcon         := LoadIcon(hInstance, 'MAINICON');
    hCursor       := LoadCursor (0, IDC_ARROW);
    hbrBackground := HBRUSH(GetStockObject(BLACK_BRUSH));
    lpszMenuName  := nil;
    lpszClassName := PChar(class_name);
  end;
RegisterClassEx(wndclass);

Result := CreateWindowEx(WS_EX_APPWINDOW,
                         PChar(class_name),
                         PChar(class_name),
                         WS_EX_APPWINDOW,
                         0, 0,
                         screen.width, screen.height,
                         0, 0, 0, nil);

SendMessage(Result, WM_SETICON, 1, LoadIcon(hInstance, 'MAINICON'));
end;

function GetTimer: integer;
var
T : LARGE_INTEGER;
F : LARGE_INTEGER;
begin
QueryPerformanceFrequency(Int64(F));
QueryPerformanceCounter(Int64(T));
result := round(1000 * T.QuadPart / F.QuadPart);
end;

function ProgramInitialization : boolean;
begin
result := false;
Handle := CreateWindow;
Console.Write('Window');
if (Handle = 0) or (not oglInitialization) then exit;
Console.Write('OpenGL');
SetForegroundWindow(Handle);
ShowWindow(Handle, SW_SHOW);
ShowCursor(false);
randomize;
ScreenPrepare(@screen);
OptionsLoad;
ScreenSet(@screen);
result := true;
Console.Write('Screen');
end;

procedure ProgramDeInitialization;
begin
OptionsSave;
screen.fullscreen := false;
ScreenSetMode(@screen);
oglDeinitialization;
end;

end.
