unit e_textures;

interface

uses
  Windows, OpenGL, e_types, e_utils, pngimage;

procedure SetTexture(texture : gluint);
function  EmptyTexture(size : integer; smooth : boolean) : gluint;

procedure SetSphericalTexGen(spherical : boolean);

procedure TexturePushMatrix;
procedure TexturePopMatrix;
procedure TextureTranslate(x, y : single);
procedure TextureScale(x, y : single);
procedure TextureRotate(x, y : single);

function  LoadPNGTexture(filename: String) : gLuint;
procedure TexturesInitialize;

type
  PByteArray = ^TByteArray;
  TByteArray = array[0..32767] of Byte;

const
  GL_CLAMP_TO_EDGE = $812F;  
  texture_machine_frames = 10;

var
  texture_machine : array[1..texture_machine_frames] of gluint;
  texture_deeper, texture_map, texture_cursor,
  texture_bonus_back, texture_bonus_score1, texture_bonus_score2,
  texture_stone1, texture_stone2,
  texture_arrow, texture_line, texture_trench,
  texture_explosion, texture_dust, texture_debris : gluint;

  texture_time             : single = 0;

implementation

function  gluBuild2DMipmaps(Target: GLenum; Components, Width, Height: GLint; Format, atype: GLenum; Data: Pointer): GLint; stdcall; external glu32;
procedure glGenTextures(n: GLsizei; var textures: GLuint); stdcall; external opengl32;
procedure glBindTexture(target: GLenum; texture: GLuint); stdcall; external opengl32;

procedure SetTexture;
begin
glBindTexture(GL_TEXTURE_2D, texture);
end;

function EmptyTexture(size : integer; smooth : boolean) : gluint;
var
txtnumber : glUint;
pData : Pointer;
begin
GetMem(pData, size * size * 4);

glGenTextures(1, txtnumber);
SetTexture(txtnumber);
glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, size, size, 0,
             GL_RGBA, GL_UNSIGNED_BYTE, pData);

  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);

result :=txtNumber;
end;   

procedure SetSphericalTexGen;
begin
glTexGeni(GL_S, GL_TEXTURE_GEN_MODE, GL_SPHERE_MAP);
glTexGeni(GL_T, GL_TEXTURE_GEN_MODE, GL_SPHERE_MAP);
if spherical then
  begin
    glEnable(GL_TEXTURE_GEN_S);
    glEnable(GL_TEXTURE_GEN_T);
  end
else
  begin
    glDisable(GL_TEXTURE_GEN_S);
    glDisable(GL_TEXTURE_GEN_T);
  end;
end;

procedure TexturePushMatrix;
begin
glMatrixMode(GL_TEXTURE);
glPushMatrix;
glMatrixMode(GL_MODELVIEW);
end;

procedure TexturePopMatrix;
begin
glMatrixMode(GL_TEXTURE);
glPopMatrix;
glMatrixMode(GL_MODELVIEW);
end;

procedure TextureTranslate;
begin
glMatrixMode(GL_TEXTURE);
glTranslatef(x, y, 0);
glMatrixMode(GL_MODELVIEW);
end;

procedure TextureScale;
begin
glMatrixMode(GL_TEXTURE);
glScalef(x, y, 1);
glMatrixMode(GL_MODELVIEW);
end;

procedure TextureRotate;
begin
glMatrixMode(GL_TEXTURE);
glRotatef(x, 1, 0, 0);
glRotatef(y, 0, 1, 0);
glMatrixMode(GL_MODELVIEW);
end;

function CreatePNGTexture(Width, Height: Word; pData: Pointer): Integer;
var
  Texture: GLuint;
begin
  glGenTextures(1, Texture);
  glBindTexture(GL_TEXTURE_2D, Texture);
  glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);  {Texture blends with object background}
  //glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_DECAL);  {Texture does NOT blend with object background}

  { Select a filtering type. BiLinear filtering produces very good results with little performance impact
    GL_NEAREST               - Basic texture (grainy looking texture)
    GL_LINEAR                - BiLinear filtering
    GL_LINEAR_MIPMAP_NEAREST - Basic mipmapped texture
    GL_LINEAR_MIPMAP_LINEAR  - BiLinear Mipmapped texture
  } 

  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR_MIPMAP_LINEAR); { only first two can be used }
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR); { all of the above can be used }

  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);  

  gluBuild2DMipmaps(GL_TEXTURE_2D, GL_RGBA, Width, Height, GL_RGBA, GL_UNSIGNED_BYTE, pData);

  result := Texture;
end;

function LoadPNGTexture(Filename: String) : GLuint;
const
  LoadFromResource = false;
var
  Data : Array of DWORD;
  W, Width : Integer;
  H, Height : Integer;
  AlphaPtr: PByte;
  PNG : TPngObject;
  Texture : gluint;
begin
  if not FileExists(filename) then exit;

  PNG := TPngObject.Create;
  PNG.LoadFromFile(Filename);

  if PNG.Header.ColorType = COLOR_PALETTE then
    begin
      PNG.free;
      exit;
    end;

  Width := PNG.Width;
  Height := PNG.Height;
  SetLength(Data, Width * Height);

  if (PNG.Header.ColorType = COLOR_RGB) or
     (PNG.Header.ColorType = COLOR_GRAYSCALE) then
  begin
    For H := 0 to Height - 1 do
      For W := 0 to Width - 1 do
        Data[W + (H * Width)] := $FF000000 or PNG.Pixels[W, (Height-1)-H];
  end else
  if (PNG.Header.ColorType = COLOR_RGBALPHA) or
     (PNG.Header.ColorType = COLOR_GRAYSCALEALPHA) then
  begin
    for H := 0 to Height - 1 do
    begin
      AlphaPtr := PByte(PNG.AlphaScanline[(Height-1)-H]);
      for W := 0 to Width - 1 do
      begin
        Data[W + (H * Width)] := (($FF000000 and (AlphaPtr^ shl 24)) or PNG.Pixels[W, (Height-1)-H]);
        Inc(AlphaPtr);
      end;
    end;
  end;

  PNG.free;

  Texture := CreatePNGTexture(Width, Height, @Data[0]);

  Result := Texture;
end;

procedure TexturesInitialize;
var
i : integer;
begin
texture_deeper            := LoadPNGTexture('textures/deeper.png');
texture_cursor            := LoadPNGTexture('textures/cursor.png');
texture_map               := LoadPNGTexture('textures/map.png');

for i := 1 to texture_machine_frames do
  texture_machine[i] := LoadPNGTexture('textures/machine/machine000' + inttostr(i - 1) + '.png');

texture_arrow             := LoadPNGTexture('textures/arrow.png');
texture_line              := LoadPNGTexture('textures/line.png');
texture_trench            := LoadPNGTexture('textures/trench.png');

texture_explosion         := LoadPNGTexture('textures/explosion.png');
texture_dust              := LoadPNGTexture('textures/dust.png');
texture_debris            := LoadPNGTexture('textures/debris.png');

texture_bonus_back        := LoadPNGTexture('textures/bonuses/back.png');
texture_bonus_score1      := LoadPNGTexture('textures/bonuses/score1.png');
texture_bonus_score2      := LoadPNGTexture('textures/bonuses/score2.png');

texture_stone1            := LoadPNGTexture('textures/stones/stone1.png');
texture_stone2            := LoadPNGTexture('textures/stones/stone2.png');
end;

end.
