unit g_Map;

interface
uses
  Classes,
  HGE,HGESprite,HGEParticle,
  g_math,g_cfg,g_utils;

type
  TField = class
    mood : byte;
    x,y : single;
    Visual : boolean;
    Busy : boolean;
    particle : IHGEParticleSystem;
    Color : cardinal;
    Scale : single;
    constructor Create;
  end;

  TFieldManager = class
    Field : array [0..16,0..16] of TField;
    Tex : array [0..6] of ITexture;
    Spr : array [0..6] of IHGESprite;
    TexParticle : ITexture;
    SelectField : pointer;
    constructor Create;
    procedure Clear;
    procedure Render;
    procedure Update;
    procedure MakeBusy(i,j : word);
    procedure MakeFieldVisual(id1,id2 : word);
    function CheckEnd : boolean;
  end;

  TMap = class
    public
      Name : string;                                    //  
      Description : PChar;                              // 
      Tex : ITexture;
      Spr : IHGESprite;
      Radius : single;
      levelName : string[12];
      constructor Create;
      procedure Load(FileName : string);
      procedure Render;
      procedure UpLayerRender;
      procedure Update(delta : single);
  end;

const
  FIELD_SIZE = 16;         //      

var
  FieldMan : TFieldManager;

implementation
uses
  g_Types,g_gun,g_gui;

constructor TField.Create;
begin
  inherited;
  Visual := false;
  Busy := false;
  Scale := 0.38*WndAspect;
end;

constructor TFieldManager.Create;
var
  I: Integer;
  J: Integer;
begin
  inherited;
  for I := 0 to FIELD_SIZE do
  for J := 0 to FIELD_SIZE do
    Field[I,J] := TField.Create;
  TexParticle := _HGE.Texture_Load(Cfg_ReadStr('data\cfg\map.cfg','TEXTURE_PARTICLE_FIELD','particles.png'));
end;

procedure TFieldManager.Clear;
var i,j : word;
begin
  for I := 0 to FIELD_SIZE do
  for j := 0 to FIELD_SIZE do begin
    Field[i,j].Visual := false;
    Field[i,j].Busy := false;
    Field[i,j].mood := 0;
    Field[i,j].Color := $00ffffff;
  end;
end;

procedure TFieldManager.MakeFieldVisual(id1,id2 : word);
var
  I: Integer;
  J: Integer;
begin
  for I := 0 to FIELD_SIZE do
  for J := 0 to FIELD_SIZE do begin
    if id1>0 then
    if (not Field[id1-1,id2].Busy)and(Field[id1-1,id2].mood>0) then Field[id1-1,id2].Visual := true;
    if id1<FIELD_SIZE then
    if (not Field[id1+1,id2].Busy)and(Field[id1+1,id2].mood>0) then Field[id1+1,id2].Visual := true;
    if id2>0 then
    if (not Field[id1,id2-1].Busy)and(Field[id1,id2-1].mood>0) then Field[id1,id2-1].Visual := true;
    if id2<FIELD_SIZE then
    if (not Field[id1,id2+1].Busy)and(Field[id1,id2+1].mood>0) then Field[id1,id2+1].Visual := true;
  end;
end;

procedure TFieldManager.Render;
var i : word;
  j: Integer;
begin
  for i := 0 to FIELD_SIZE do
  for j := 0 to FIELD_SIZE do
    if Field[i,j].Visual then begin
      Spr[Field[i,j].mood].SetColor(Field[i,j].Color);
      Spr[Field[i,j].mood].RenderEx(Field[i,j].x,Field[i,j].y,0,Field[i,j].Scale)
    end;
  if SelectField <> nil then begin
    Spr[TField(SelectField^).mood].SetColor(TField(SelectField^).Color);
    Spr[TField(SelectField^).mood].RenderEx(TField(SelectField^).x,TField(SelectField^).y,0,TField(SelectField^).Scale)
  end;
end;

procedure TFieldManager.Update;
var i : word;
    j: Integer;
    tX,tY : single;
begin
  SelectField :=nil;
  _HGE.Input_GetMousePos(tX,tY);
  for i := 0 to FIELD_SIZE do
  for j := 0 to FIELD_SIZE do
    if Field[i,j].Visual then begin
      if Field[i,j].Color shr 24 < $ff then Field[i,j].Color := Field[i,j].Color + $03000000;
      if distance2d(tx,ty,Field[i,j].X,Field[i,j].Y) < 12*WndAspect then begin
        SelectField := @Field[i,j];
        if Field[i,j].Scale < 0.45*WndAspect then Field[i,j].Scale := Field[i,j].Scale+0.01;
      end
      else if Field[i,j].Scale > 0.38*WndAspect then Field[i,j].Scale := Field[i,j].Scale-0.01;
    end;
end;

procedure TFieldManager.MakeBusy(i,j : word);
begin
  Field[i,j].Visual := false;
  Field[i,j].Busy := true;
  MakeFieldVisual(i,j);
  ToggleParSys(Field[i,j].Particle,'data\particles\particle_field_make_busy.psi',TexParticle,64,0,32,
               round(Field[i,j].X),round(Field[i,j].Y),
               BLEND_ALPHABLEND or BLEND_COLORMUL or BLEND_ALPHAADD,$ffffffff,
               Game.GameplayColorAll[Field[i,j].mood]);
  if CheckEnd then LevelManager.LevelEnd;
end;

function TFieldManager.CheckEnd : boolean;
var
  i : byte;
  j : byte;
begin
  result := true;
  for i := 0 to FIELD_SIZE do
  for j := 0 to FIELD_SIZE do
    if (Field[i,j].mood>0)and(not Field[i,j].Busy) then Result := false;
end;

constructor TMap.Create;
var i : byte;
begin
  inherited;
  Radius := 475*WndAspect;
  Tex := _HGE.Texture_Load(Cfg_ReadStr('data\cfg\map.cfg','TEXTURE_BACK','data\graphics\back.png'));
  Spr := THGESprite.Create(Tex,0,0,Tex.GetWidth,Tex.GetHeight);
  Spr.SetHotSpot(Tex.GetWidth/2,Tex.GetHeight/2);
  Spr.SetColor($ffffffff);
  FieldMan := TFieldManager.Create;
  for I := 0 to 6 do begin
    FieldMan.Tex[i] := _HGE.Texture_Load('data\graphics\box'+inttostr(i+1)+'.png');
    FieldMan.Spr[i] := THGESprite.Create(FieldMan.Tex[i],0,0,FieldMan.Tex[i].GetWidth,FieldMan.Tex[i].GetHeight);
    FieldMan.Spr[i].SetHotSpot(FieldMan.Tex[i].GetWidth / 2,FieldMan.Tex[i].GetHeight / 2);
    FieldMan.Spr[i].SetColor($ccffffff);
  end;
end;

procedure TMap.Load(FileName : string);
var
  f : file of byte;
  count,i : word;
  tempX,tempY : byte;
begin
  FieldMan.Clear;
  if not FileExists(FileName) then begin
     _HGE.System_Log(FileName + ' - file not found');
    exit;
  end;
  AssignFile(f,FileName);
  Reset(f);
  BlockRead(f,LevelName,12);
  BlockRead(f,Count,2);
  for I := 0 to Count - 1 do begin
    BlockRead(F,tempX,1);
    BlockRead(F,tempY,1);
    FieldMan.Field[tempX,tempY].X := tempX*24*WndAspect+206*WndAspect;
    FieldMan.Field[tempX,tempY].Y := tempY*24*WndAspect+120*WndAspect;
    BlockRead(F,FieldMan.Field[tempX,tempY].mood,1);
  end;

  //   
  if FieldMan.Field[8,8].mood > 0 then FieldMan.Field[8,8].Visual := true;
  if FieldMan.Field[7,8].mood > 0 then FieldMan.Field[7,8].Visual := true;
  if FieldMan.Field[9,8].mood > 0 then FieldMan.Field[9,8].Visual := true;
  if FieldMan.Field[8,7].mood > 0 then FieldMan.Field[8,7].Visual := true;
  if FieldMan.Field[8,9].mood > 0 then FieldMan.Field[8,9].Visual := true;

  CloseFile(f);
  CHECK_GAMEPLAY_COLOR;
  GUI.HintLevelName.Add(SCR_CENTER_W,SCR_CENTER_H div 2,levelName,Game.GameplayColor[Random(Game.GameplayColorCount)]);
end;

procedure TMap.Render;
begin
  Spr.RenderStretch(0,0,800*WndAspect,600*WndAspect);
  FieldMan.Render;
end;

procedure TMap.UpLayerRender;
begin
  //Spr2.RenderStretch(0,0,800,600);
end;

procedure TMap.Update(delta : single);
begin
  FieldMan.Update;
end;

end.
