unit g_gun;

interface
uses
  Classes,
  HGE,HGESprite,HGEParticle,
  g_Types,g_math,g_cfg,g_map,g_gui;

type
  TBall = class
    startX,startY,X,Y,moveX,moveY : Single;
    Speed : single;
    Angle : single;
    Color : Cardinal;
    active : boolean;
    Radius : single;
    move : boolean;
    particle : IHGEParticleSystem;
    procedure Stop;
    procedure SetMovePos(const toX,toY : single);
    procedure MoveTo;
  end;

  TBallManager = class
    Tex : ITexture;
    Spr : IHGESprite;
    TexParticle : ITexture;
    List : TList;
    CurrentBall : integer;
    constructor Create;
    procedure Init;
    procedure AddBall(const _X,_Y,_Angle : single; _Color : cardinal);
    procedure Dead(id : word);
    procedure Clear;
    procedure CheckGameplayColor;
    procedure Update(const delta : single);
    procedure Render;
  end;

  TGun = class
    Tex,TargetTex : ITexture;
    Spr,TargetSpr : IHGESprite;
    startX,startY : single;
    X,Y : single;
    Angle,AnglePos : single;
    Color : Cardinal;
    inShot : boolean;
    FireTime : single;
    FireSpeed : single;
    Distance : single;
    particle_L,particle_R : IHGEParticleSystem;
    constructor Create;
    procedure Init(_X, _Y: single);
    procedure Shot;
    procedure Update(const delta : single);
    procedure Render;
  end;

var
  BallMan : TBallManager;
  Gun : TGun;

procedure CHECK_GAMEPLAY_COLOR;

implementation

procedure CHECK_GAMEPLAY_COLOR;
var
  I: Integer;
  flag : boolean;
begin
  BallMan.CheckGameplayColor;
  flag := true;
  for I := 0 to Game.GameplayColorCount - 1 do
  if Game.GameplayColor[i] = Gun.Color then begin
    flag := false;
    break;
  end;
  if flag then Gun.Color := Game.GameplayColor[random(Game.GameplayColorCount)];
end;

procedure TBall.Stop;
begin
  active := false;
end;

procedure TBall.MoveTo;
begin
  aspire(X,moveX,Distance2D(X,Y,moveX,moveY)/10);
  aspire(Y,moveY,Distance2D(X,Y,moveX,moveY)/10);
  if (abs(X-moveX)<0.01)and(abs(Y-moveY)<0.01) then move := false;
end;

constructor TBallManager.Create;
begin
  inherited;
  Tex := _HGE.Texture_Load(Cfg_ReadStr('data\cfg\player.cfg','TEXTURE_BALL','ball.png'));
  Spr := THGESprite.Create(Tex,0,0,Tex.GetWidth,Tex.GetHeight);
  Spr.SetHotSpot(Tex.GetWidth / 2,Tex.GetHeight / 2);
  TexParticle := _HGE.Texture_Load(Cfg_ReadStr('data\cfg\player.cfg','TEXTURE_PARTICLE_BALL','particles.png'));
  CurrentBall := -1;
  List := TList.Create;
end;

procedure TBallManager.Init;
begin
  Clear;
end;

procedure TBallManager.AddBall(const _X,_Y,_Angle : single; _Color: Cardinal);
begin
  CurrentBall := List.Add(TBall.Create);
  with TBall(List.Items[CurrentBall]) do begin
    Angle := _Angle;
    Color := _Color;
    startX := _X;
    startY := _Y;
    X := _X+Radius;
    Y := _Y+Radius;
    moveX := X;
    MoveY := Y;
    Speed := 6*WndAspect;
    active := true;
    Radius := 12*WndAspect;
    move := false;
  end;
end;

procedure TBallManager.Dead(id : word);
begin
  ToggleParSys(TBall(List.Items[id]).Particle,'data\particles\particle_collision.psi',TexParticle,64,0,32,
               round(TBall(List.Items[id]).X),round(TBall(List.Items[id]).Y),
               BLEND_ALPHABLEND or BLEND_COLORMUL or BLEND_ALPHAADD,$ffffffff,
               TBall(List.Items[id]).Color);
  TBall(List.Items[id]).Free;
  List.Delete(id);
end;

procedure TBallManager.Clear;
var i : word;
begin
  List.Clear;
  i := 0;
  while i < List.Count do begin
    TBall(List.Items[i]).Free;
    inc(i);
  end;
end;

procedure TBall.SetMovePos(const toX,toY : single);
begin
  move := true;
  moveX := toX;
  moveY := toY;
end;

procedure TBallManager.CheckGameplayColor;
var
  i,j,m : integer;
  flag : boolean;
  tField : array [0..99] of Cardinal;
  temp : Cardinal;
  incr : integer;
begin
  m := 0;
  for I := 0 to FIELD_SIZE do
   for j := 0 to FIELD_SIZE do
    if (FieldMan.Field[i,j].Visual) then begin//and(Game.GameplayColorAll[FieldMan.Field[i,j].mood]=_Color) then begin
      tField[m] := Game.GameplayColorAll[FieldMan.Field[i,j].mood];
      inc(m);
    end;

  //   
  Game.GameplayColor[0]:=tField[0];
  Game.GameplayColorCount:=1;

  for i := 0 to m-1 do begin
    flag := true;
    j := 0;
    while j < Game.GameplayColorCount do begin
      if tField[i]=Game.GameplayColor[j] then begin
        flag := false;
        break;
      end;
      inc(j);
    end;
    if flag then begin
      Game.GameplayColor[j]:= tField[i];
      inc(Game.GameplayColorCount);
    end;
  end;

   //***Sort***\\
 {incr := Game.GameplayColorCount-1 div 2;
 while incr>0 do begin
   for i := incr to Game.GameplayColorCount - 1 do begin
     j := i - incr;
     while(j>=0) do begin
       if Game.GameplayColor[j]<Game.GameplayColor[j+incr] then begin
         temp := Game.GameplayColor[j];
         Game.GameplayColor[j] := Game.GameplayColor[j+incr];
         Game.GameplayColor[j+incr] := temp;
         j := j-incr;
       end else j :=-1;
     end;
   end;
   incr := incr div 2;
 end;   }
end;

procedure TBallManager.Update(const delta : single);
var i,m,j,k : word;
begin
  j:=0;
  while j<List.Count do begin
    with TBall(List.Items[j]) do
    if active then begin
      X := X + Speed*cos(deg2rad*Angle);
      Y := Y + Speed*sin(deg2rad*Angle);
      if Distance2D(startX,startY,X,Y) > Map.Radius then Dead(j) else begin
        for i := 0 to FIELD_SIZE do
          for m := 0 to FIELD_SIZE do begin
          if FieldMan.Field[i,m].Visual then
          if 12*WndAspect > Distance2D(X,Y,FieldMan.Field[i,m].X,FieldMan.Field[i,m].Y) then begin
            SetMovePos(FieldMan.Field[i,m].X,FieldMan.Field[i,m].Y);
            Stop;
            for k := 0 to List.Count - 1 do
              if (k<>j)and(distance2d(X,Y,TBall(List.Items[k]).X,TBall(List.Items[k]).Y)<12*WndAspect) then begin
                TBall(List.Items[k]).Free;
                List.Delete(k);
                break;
              end;
            if Game.GameplayColorAll[FieldMan.Field[i,m].mood] = Color
             then begin
               FieldMan.MakeBusy(i,m);
               CHECK_GAMEPLAY_COLOR;
               inc(Profile.tScore,300);
               gui.GamePlayHint.Add(X,Y,'+300',Color);
             end else begin
               dec(Profile.tScore,100);
               gui.GamePlayHint.Add(X,Y,'-100',Color);
             end;
            break;
          end;
         end;
        end;
      end else if move then MoveTo;
    inc(j);
  end;
end;

procedure TBallManager.Render;
var i : word;
begin
  i := 0;
  while i < List.Count do begin
    with TBall(List.Items[i]) do begin
      //if not active then exit;
      Spr.SetColor(Color);
       Spr.RenderEx(X,Y,deg2rad*Angle,0.32*WndAspect);
    end;
  inc(i);
  end;
end;

constructor TGun.Create;
begin
  BallMan := TBallManager.Create;
  Tex := _HGE.Texture_Load(Cfg_ReadStr('data\cfg\player.cfg','TEXTURE_GUN','gun.png'));
  Spr := THGESprite.Create(Tex,0,0,Tex.GetWidth,Tex.GetHeight);
  Spr.SetHotSpot(Tex.GetWidth / 2,Tex.GetHeight / 2);
  Color := $ffffffff;
end;

procedure TGun.Init(_X, _Y : single);
begin
  X := 0;
  Y := 0;
  startX := _X;
  startY := _Y;
  //CHECK_GAMEPLAY_COLOR;
  //Color := Game.GameplayColor[random(Game.GameplayColorCount)];
  inShot := false;
  FireTime := 0;
  FireSpeed := 0.25;
  Distance := 235*WndAspect;
  ToggleParSys(Particle_L,'data\particles\particle_gun.psi',Game.TexParticle,96,64,32,0,0,
               BLEND_ALPHAADD,$aa33aaee);
  ToggleParSys(Particle_R,'data\particles\particle_gun.psi',Game.TexParticle,96,64,32,0,0,
               BLEND_ALPHAADD,$aa33aaee);
  BallMan.Init;
  ToggleParSys(GUI.Particle,'data\particles\particle_speed.psi',Game.TexParticle,64,0,32,0,0,
               BLEND_ALPHABLEND or BLEND_COLORMUL or BLEND_ALPHAADD,Color);
end;

procedure TGun.Shot;
begin
  BallMan.AddBall(X,Y,Angle,Color);
end;

procedure TGun.Update(const delta : single);
var
 tX,tY,tAngle : single;
begin
  FireTime := FireTime + delta;
  if (inShot) and (FireTime > FireSpeed) then begin
    Shot;
    inShot := false;
    Color := Game.GameplayColor[random(Game.GameplayColorCount)];
    //  
    ToggleParSys(GUI.Particle,'data\particles\particle_speed.psi',Game.TexParticle,64,0,32,0,0,
               BLEND_ALPHABLEND or BLEND_COLORMUL or BLEND_ALPHAADD,Color);
    FireTime := 0;
  end else inShot := false;
  _HGE.Input_GetMousePos(tX,tY);
  Angle :=  GetAngle(X,Y,startX,startY);
  AngleNorm(Angle);
  tAngle := GetAngle(tX,tY,startX,startY);
  AngleNorm(tAngle);
  GUI.CursorAngle := Angle+135;
  if (tAngle - AnglePos) > 180 then AnglePos := AnglePos + 360;
  if (AnglePos - tAngle) > 180 then AnglePos := AnglePos - 360;
  AnglePos := AnglePos + (tAngle - AnglePos)/10;
  X := startX - distance*cos(deg2rad*AnglePos);
  Y := startY - distance*sin(deg2rad*AnglePos);
  BallMan.Update(delta);
  if particle_L<>nil then
    particle_L.MoveTo(X+25*WndAspect*cos((Angle+135)*deg2rad),Y+25*WndAspect*sin((Angle+135)*deg2rad));
  if particle_R<>nil then
    particle_R.MoveTo(X+25*WndAspect*cos((Angle+225)*deg2rad),Y+25*WndAspect*sin((Angle+225)*deg2rad));
end;

procedure TGun.Render;
begin
  BallMan.Render;
  Spr.SetColor(Color);
  Spr.RenderEx(X,Y,deg2rad*Angle,0.5*WndAspect);
end;

end.
