unit g_Main;

interface
uses
  windows,messages,HGE,HGEFont,HGESprite,
  g_Types,g_Game,g_GUI,g_Menu,g_levelManager,g_Intro,g_Loading,g_Options,g_gun;

function RestoreTargets : boolean;
function FocusLost : boolean;
function FocusReturn : boolean;

function MainUpdate : boolean;
function MainDraw : boolean;
procedure MainInit;
procedure MainDestroy;
implementation

uses g_Map;

function RestoreTargets : boolean;
begin
  result := false;
  {with map do begin
  //Target._Release;
  //Target := _HGE.Target_Create(fullRadius*2,fullRadius*2,false);
  SprWar := THGESprite.Create(_HGE.Target_GetTexture(target),0,0,fullRadius*2,fullRadius*2);
  _HGE.Gfx_BeginScene(Target);
  Spr.RenderStretch(X,Y,fullRadius*2,fullRadius*2);
  _HGE.Gfx_EndScene;
  end;
  //  _HGE.Target_GetTexture(Map.Target) <> nil then result := true; }
end;

function FocusLost : boolean;
begin
  case Game.State of
    gsGame : Game.Channel.Pause;
    gsMenu :  Menu.Channel.Pause;
  end;
  Game.Pause := true;
  result := true;
end;

function FocusReturn : boolean;
begin
  case Game.State of
    gsGame : Game.Channel.Resume;
    gsMenu :  Menu.Channel.Resume;
  end;
  result := true;
end;

procedure MainInit;
begin
  randomize;
  Loading := TLoadingManager.Create;

  Loading.Process('Menu...');                                //    
  GUI := TGUI.Create;
  Menu := TMenu.Create;

  Loading.Process('Game...');
  LevelManager := TLevelManager.Create;

  Loading.Process('Complete...');
  Menu.Activate;
  //levelManager.GameToTime;
  //game.State := gsGame;
  GAME_GLOBAL__TIME_OLD := _HGE.Timer_GetTime - GAME_GLOBAL_SPEED;
end;

procedure MainControl;
begin
  if _HGE.Input_KeyUp(HGEK_ESCAPE) then begin
    if Game.State = gsIntro then IntroComplite else
      if Game.State = gsGame then begin
        Game.Channel.Pause;
        Menu.Activate;
      end;
  end;

  if _HGE.Input_KeyDown(HGEK_F12) then _HGE.System_Snapshot('');
  if _HGE.Input_KeyDown(HGEK_P) then
  if Game.State=gsGame then Game.Pause := not Game.Pause
  else
  else
    if (Game.State=gsGame)and(_HGE.Input_KeyDown(HGEK_LBUTTON)) then
      Gun.inShot := true;
end;

procedure GameUpdate(const delta : single);
begin
  LevelManager.Update(delta);
end;

function MainUpdate : boolean;
var wTime,wTime_Delta : single;
begin
  MainControl;

  if  Game.State = gsMenu then Menu.Update(_HGE.Timer_GetDelta);

  wTime       := _HGE.Timer_GetTime;
  wTime_Delta := wTime - GAME_GLOBAL__TIME_OLD;
  while wTime_Delta >= GAME_GLOBAL_SPEED do
    begin
      case Game.State of
        gsIntro : IntroUpdate(GAME_GLOBAL_SPEED);
        gsLoading : Loading.WaitPressAnyKey;
        gsGame : GameUpdate(GAME_GLOBAL_SPEED);
      end;
      wTime_Delta := wTime_Delta-GAME_GLOBAL_SPEED;
    end;
  GAME_GLOBAL__TIME_OLD := wTime - wTime_Delta;

  Result := Game.Finish;
end;

procedure GameDraw;
begin
  //_HGE.Gfx_Clear($ffffffff);
  World_Draw;
  GUI.GameRender;
end;

function MainDraw : boolean;
begin
  _HGE.Gfx_BeginScene;
  case Game.State of
    gsIntro : IntroDraw;
    gsMenu : Menu.Draw;
    gsGame : GameDraw();
  end;
  _HGE.Gfx_EndScene;
  result := false;
end;

procedure MainDestroy;
begin
  case Game.State of
    gsMenu : Menu.Destroy;
    gsGame : begin
      Menu.Destroy;
      World_Destroy;
      Loading.Destroy;
    end;
  end;
end;

end.


