unit g_math;

interface
uses
  windows;

const
 rad2deg = 180/pi;
 deg2rad = pi/180;
 Pi2 = Pi*2;

// Trigonometry
function ArcTan(Y, X: Extended): Extended;
function ArcTan2(const Y, X: single): single;
function ArcSin(const X: Extended): Extended;
function ArcCos(const X: single): single;

procedure AngleNorm(var angle : single);
procedure AngleNormPi(var angle : single);

function Pow(x : Single; n : Integer) : single;

// Vector math
function Distance2D(const X1,Y1,X2,Y2 : single) : single;
function distance2D_quad(const X1,Y1,X2,Y2 : single) : single;
function DistancePointToLine(X,Y : single; startX,startY, endX,endY : single) : single;

function GetAngle(X1,Y1,X2,Y2 : single) : single;

  // S  AIM   SPEED   SIGN
procedure Aspire(var s : single; aim, speed : single);

procedure NormalizeLength(var X,Y : single;const l : single);
implementation

function ArcTan(Y, X: Extended): Extended;
asm
        FLD     Y
        FLD     X
        FPATAN
        FWAIT
end;

function ArcTan2(const Y, X: single): single;
asm
 FLD     Y
 FLD     X
 FPATAN
 FWAIT
end;

function ArcSin(const X: Extended): Extended;
begin
  Result := ArcTan(X, Sqrt(1 - X * X))
end;

function ArcCos(const X: single): single;
begin
if abs(X) > 1 then
 Result := 0
else
 Result := ArcTan2(Sqrt(1 - X * X), X);
end;

function Pow(x : Single; n : Integer) : single;
begin
  if n < 0 then result := -1;
  case n of
    0 : result := 1;
    1 : result := x;
  else result := x*Pow(x,n-1);
  end;
end;

procedure AngleNorm(var angle : single);
begin
  if angle > 360 then angle := angle - 360 else
  if angle < 0   then angle := angle + 360;
end;

procedure AngleNormPi(var angle : single);
begin
  if Angle > Pi2 then Angle := Angle - Pi2 else
  if Angle < 0   then Angle := Angle + Pi2;
end;

function Distance2D(const X1,Y1,X2,Y2 : single) : single;
var X,Y : single;
begin
  X := X1-X2;
  Y := Y1-Y2;
  result := sqrt(X*X+Y*Y)
end;

function distance2D_quad(const X1,Y1,X2,Y2 : single) : single;
var X,Y : single;
begin
  X := X1-X2;
  Y := Y1-Y2;
  if ( X < 0 ) then X := -X;
  if ( Y < 0 ) then Y := -Y;
  if ( X < Y ) then result := Y
  else result := X;
end;


function DistancePointToLine(X,Y : single; startX,startY, endX,endY : single) : single;
var
  LineMag, U : single;
  IntersectionX : single;
  IntersectionY : single;
begin
  Result := 99999999;

  LineMag := Distance2D(startX,startY, endX,endY);

  U := (((X-startX)*(endX-startX))+
       ((Y-startY)*(endY-startY)))/(LineMag*LineMag);

  if(U < 0.0) or (U > 1.0) then exit;

  IntersectionX := startX+U*(endX-startX);
  IntersectionY := startY+U*(endY-startY);

  Result := Distance2D(X,Y, IntersectionX,IntersectionY);
end;


function GetAngle(X1,Y1,X2,Y2 : single) : single;
begin
  Result := ArcTan2((Y2-Y1),(X2-X1))*180/3.14;
end;

{function GetAngle(X1,Y1,X2,Y2 : single) : single;
var x,y,s : single;
begin
  x := x2 - x1;
  y := y2 - y1;
  s := sqrt(sqr(x) + sqr(y));
  if s <>0 then result := rad2deg*(arcsin(x / s)) else result := 0;
  if y < 0 then result := 180 - result;
end;    }

procedure Aspire(var s : single; aim, speed : single);
var
b : boolean;
begin
b := true;

if (s + speed < aim) then begin s := s + speed; b := false; end;
if (s - speed > aim) then begin s := s - speed; b := false; end;

if b then s := aim;
end;

procedure NormalizeLength(var X,Y : single;const l : single);
var
  s : single;
begin
  s := sqrt(X*X + Y*Y) / l;
  X := X / s;
  Y := Y / s;
end;

end.
